@@ -150,6 +150,110 @@ module Kramdown
         generate_id(value).gsub(/-+/, '-')
       end
 
+      SVG_COLORS = Hash.new {|h, k| k}
+      <<COLORS.each_line {|l| k, v = l.chomp.split; SVG_COLORS[k] = v}
+black	#000000
+silver	#C0C0C0
+gray	#808080
+white	#FFFFFF
+maroon	#800000
+red	#FF0000
+purple	#800080
+fuchsia	#FF00FF
+green	#008000
+lime	#00FF00
+olive	#808000
+yellow	#FFFF00
+navy	#000080
+blue	#0000FF
+teal	#008080
+aqua	#00FFFF
+COLORS
+
+      def svg_munch_id(id)
+        id.gsub(/[^-._A-Za-z0-9]/) {|x| "_%02X" % x.ord}
+      end
+
+      def self.hex_to_lin(h)
+        h.to_i(16)**2.22        # approximating sRGB gamma
+      end
+      define_method :hex_to_lin, &method(:hex_to_lin)
+
+      B_W_THRESHOLD = hex_to_lin("a4") # a little brighter than 1/2 0xFF -> white
+
+      def svg_munch_color(c, fill)
+        c = SVG_COLORS[c]
+        case c
+        when /\A#(..)(..)(..)\z/
+          if hex_to_lin($1)*0.2126 + hex_to_lin($2)*0.7152 + hex_to_lin($3)*0.0722 >= B_W_THRESHOLD
+            'white'
+          else
+            'black'
+          end
+        when 'none'
+          'none' if fill        # delete for stroke
+        else
+          c
+        end
+      end
+
+      def svg_clean(s)          # expensive, risky
+        require "rexml/document"
+        d = REXML::Document.new(s)
+        REXML::XPath.each(d.root, "//*[@shape-rendering]") { |x| x.attributes["shape-rendering"] = nil }  #; warn x.inspect }
+        REXML::XPath.each(d.root, "//*[@text-rendering]") { |x| x.attributes["text-rendering"] = nil }  #; warn x.inspect  }
+        REXML::XPath.each(d.root, "//*[@stroke]") { |x| x.attributes["stroke"] = svg_munch_color(x.attributes["stroke"], false) }
+        REXML::XPath.each(d.root, "//*[@fill]") { |x| x.attributes["fill"] = svg_munch_color(x.attributes["fill"], true) }
+        REXML::XPath.each(d.root, "//*[@id]") { |x| x.attributes["id"] = svg_munch_id(x.attributes["id"]) }
+##      REXML::XPath.each(d.root, "//rect") { |x| x.attributes["style"] = "fill:none;stroke:black;stroke-width:1" unless x.attributes["style"] }
+        d.to_s
+      end
+
+      def memoize(meth, *args)
+        require 'digest'
+        Dir.mkdir(REFCACHEDIR) unless Dir.exists?(REFCACHEDIR)
+        kdrfc_version = Gem.loaded_specs["kramdown-rfc2629"].version.to_s.gsub('.', '_') rescue "UNKNOWN"
+        fn = "#{REFCACHEDIR}/kdrfc-#{kdrfc_version}-#{meth}-#{Digest::SHA256.hexdigest(Marshal.dump(args))[0...40]}.cache"
+        begin
+          out = Marshal.load(File.binread(fn))
+        rescue StandardError => e
+          # warn e.inspect
+          out = method(meth).call(*args)
+          File.binwrite(fn, Marshal.dump(out))
+        end
+        out
+      end
+
+      def svg_tool_process(t, result)
+        require 'tempfile'
+        file = Tempfile.new("kramdown-rfc")
+        file.write(result)
+        file.close
+        case t
+        when "goat"
+          result1, _s = Open3.capture2("goat #{file.path}", stdin_data: result);
+        when "ditaa"        # XXX: This needs some form of option-setting
+          result1, _s = Open3.capture2("ditaa #{file.path} --svg -o -", stdin_data: result);
+        when "mscgen"
+          result1, _s = Open3.capture2("mscgen -T svg -i #{file.path} -o -", stdin_data: result);
+        when "mermaid"
+          result1, _s = Open3.capture2("mmdc -i #{file.path}", stdin_data: result); #  -b transparent
+          outpath = file.path + ".svg"
+          result1 = File.read(outpath)
+          File.unlink(outpath)
+        when "plantuml", "plantuml-utxt"
+          plantuml = "@startuml\n#{result}\n@enduml"
+          result1, _s = Open3.capture2("plantuml -pipe -tsvg", stdin_data: plantuml);
+          result, _s = Open3.capture2("plantuml -pipe -tutxt", stdin_data: plantuml) if t == "plantuml-utxt"
+        end
+        # warn ["goat:", result1.inspect]
+        file.unlink
+        result1 = svg_clean(result1) unless t == "goat"
+        result1, _s = Open3.capture2("svgcheck -qa", stdin_data: result1);
+        # warn ["svgcheck:", result1.inspect]
+        [result, result1]
+      end
+
       def convert_codeblock(el, indent, opts)
         # el.attr['anchor'] ||= saner_generate_id(el.value) -- no longer in 1.0.6
         result = el.value
@@ -172,11 +276,12 @@ module Kramdown
           "#{' '*indent}<t>#{result}</t>\n"
         else
           artwork_attr = {}
+          t = nil
           if blockclass
             classes = blockclass.split(' ')
             classes.each do |cl|
               if md = cl.match(/\Alanguage-(.*)/)
-                artwork_attr["type"] = md[1] # XXX overwrite
+                t = artwork_attr["type"] = md[1] # XXX overwrite
               else
                 $stderr.puts "*** Unimplemented block class: #{cl}"
               end
@@ -192,7 +297,13 @@ module Kramdown
               artwork_attr[md[1]] = v
             end
           end
-          "#{' '*indent}<figure#{el_html_attributes(el)}><artwork#{html_attributes(artwork_attr)}><![CDATA[#{result}#{result =~ /\n\Z/ ? '' : "\n"}]]></artwork></figure>\n"
+          case t
+          when "goat", "ditaa", "mscgen", "plantuml", "plantuml-utxt", "mermaid"
+            result, result1 = memoize(:svg_tool_process, t, result)
+            "#{' '*indent}<figure#{el_html_attributes(el)}><artset><artwork #{html_attributes(artwork_attr.merge("type"=> "svg"))}>#{result1.sub(/.*?<svg/m, "<svg")}</artwork><artwork #{html_attributes(artwork_attr.merge("type"=> "ascii-art"))}><![CDATA[#{result}#{result =~ /\n\Z/ ? '' : "\n"}]]></artwork></artset></figure>\n"
+          else
+            "#{' '*indent}<figure#{el_html_attributes(el)}><artwork#{html_attributes(artwork_attr)}><![CDATA[#{result}#{result =~ /\n\Z/ ? '' : "\n"}]]></artwork></figure>\n"
+          end
         end
       end
 
@@ -425,9 +536,9 @@ module Kramdown
         fn = "#{REFCACHEDIR}/#{cachefile}"
         Dir.mkdir(REFCACHEDIR) unless Dir.exists?(REFCACHEDIR)
         f = File.stat(fn) rescue nil unless KRAMDOWN_REFCACHE_REFETCH
-        if !KRAMDOWN_OFFLINE && (!f || tn - f.ctime >= tvalid)
+        if !KRAMDOWN_OFFLINE && (!f || tn - f.mtime >= tvalid)
           if f
-            message = "renewing (stale by #{"%.1f" % ((tn-f.ctime)/86400)} days)"
+            message = "renewing (stale by #{"%.1f" % ((tn-f.mtime)/86400)} days)"
             fetch_timeout = 10 # seconds, give up quickly if just renewing
           else
             message = "fetching"
@@ -452,7 +563,8 @@ module Kramdown
                 if ENV["KRAMDOWN_DONT_VERIFY_HTTPS"]
                   options[:ssl_verify_mode] = OpenSSL::SSL::VERIFY_NONE
                 end             # workaround for OpenSSL on Windows...
-                open(url, options) do |uf|
+#               URI.open(url, **options) do |uf|          # not portable to older versions
+                OpenURI.open_uri(url, **options) do |uf|
                   s = uf.read
                   if uf.status[0] != "200"
                     warn "*** Status code #{status} while fetching #{url}"
@@ -461,7 +573,8 @@ module Kramdown
                   end
                 end
               end
-            rescue OpenURI::HTTPError, SocketError, Timeout::Error => e
+            rescue OpenURI::HTTPError, Errno::EHOSTUNREACH, Errno::ECONNREFUSED,
+                   SocketError, Timeout::Error => e
               warn "*** #{e} while fetching #{url}"
             end
           end
@@ -469,7 +582,7 @@ module Kramdown
         begin
           File.read(fn) # this blows up if no cache available after fetch attempt
         rescue Errno::ENOENT => e
-          warn "*** #{e} for ${fn}"
+          warn "*** #{e} for #{fn}"
         end
       end
 
@@ -521,7 +634,7 @@ module Kramdown
             fn = "reference.#{t}.#{n}.xml"
             sub, ttl, can_anchor = XML_RESOURCE_ORG_MAP[t]
             ttl ||= KRAMDOWN_REFCACHETTL  # everything but RFCs might change a lot
-            puts "Huh: ${fn}" unless sub
+            puts "*** Huh: #{fn}" unless sub
             url = "#{XML_RESOURCE_ORG_PREFIX}/#{sub}/#{fn}"
             if can_anchor # create anchor server-side for stand_alone: false
               url << "?anchor=#{anchor}"
@@ -531,7 +644,10 @@ module Kramdown
             to_insert.scrub! rescue nil # only do this for Ruby >= 2.1
             # this may be a bit controversial: Don't break the build if reference is broken
             if KRAMDOWN_OFFLINE
-              to_insert ||= "<reference anchor='#{anchor}'> <front> <title>*** BROKEN REFERENCE ***</title> <author> <organization/> </author> <date/> </front> </reference>"
+              unless to_insert
+                to_insert = "<reference anchor='#{anchor}'> <front> <title>*** BROKEN REFERENCE ***</title> <author> <organization/> </author> <date/> </front> </reference>"
+                warn "*** KRAMDOWN_OFFLINE: Inserting broken reference for #{fn}"
+              end
             else
               exit 66 unless to_insert # EX_NOINPUT
             end
